# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/tmp/tmpeh68xiaa'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setWindowModality(QtCore.Qt.NonModal)
        MainWindow.resize(800, 696)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/DXF2GCODE-001.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(-1, -1, -1, 2)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.mytabWidget = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mytabWidget.sizePolicy().hasHeightForWidth())
        self.mytabWidget.setSizePolicy(sizePolicy)
        self.mytabWidget.setMinimumSize(QtCore.QSize(200, 0))
        self.mytabWidget.setAutoFillBackground(False)
        self.mytabWidget.setObjectName("mytabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setSpacing(1)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.blocksCollapsePushButton = QtWidgets.QPushButton(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.blocksCollapsePushButton.sizePolicy().hasHeightForWidth())
        self.blocksCollapsePushButton.setSizePolicy(sizePolicy)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/collapse-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.blocksCollapsePushButton.setIcon(icon1)
        self.blocksCollapsePushButton.setIconSize(QtCore.QSize(24, 24))
        self.blocksCollapsePushButton.setObjectName("blocksCollapsePushButton")
        self.horizontalLayout_5.addWidget(self.blocksCollapsePushButton)
        self.blocksExpandPushButton = QtWidgets.QPushButton(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.blocksExpandPushButton.sizePolicy().hasHeightForWidth())
        self.blocksExpandPushButton.setSizePolicy(sizePolicy)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/expand-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.blocksExpandPushButton.setIcon(icon2)
        self.blocksExpandPushButton.setIconSize(QtCore.QSize(24, 24))
        self.blocksExpandPushButton.setObjectName("blocksExpandPushButton")
        self.horizontalLayout_5.addWidget(self.blocksExpandPushButton)
        spacerItem = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.entitiesTreeView = TreeView(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.entitiesTreeView.sizePolicy().hasHeightForWidth())
        self.entitiesTreeView.setSizePolicy(sizePolicy)
        self.entitiesTreeView.setObjectName("entitiesTreeView")
        self.verticalLayout_3.addWidget(self.entitiesTreeView)
        self.mytabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_5.setContentsMargins(5, 5, 5, 5)
        self.verticalLayout_5.setSpacing(1)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setSpacing(1)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.layersCollapsePushButton = QtWidgets.QPushButton(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersCollapsePushButton.sizePolicy().hasHeightForWidth())
        self.layersCollapsePushButton.setSizePolicy(sizePolicy)
        self.layersCollapsePushButton.setIcon(icon1)
        self.layersCollapsePushButton.setIconSize(QtCore.QSize(24, 24))
        self.layersCollapsePushButton.setObjectName("layersCollapsePushButton")
        self.horizontalLayout_4.addWidget(self.layersCollapsePushButton)
        self.layersExpandPushButton = QtWidgets.QPushButton(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersExpandPushButton.sizePolicy().hasHeightForWidth())
        self.layersExpandPushButton.setSizePolicy(sizePolicy)
        self.layersExpandPushButton.setIcon(icon2)
        self.layersExpandPushButton.setIconSize(QtCore.QSize(24, 24))
        self.layersExpandPushButton.setObjectName("layersExpandPushButton")
        self.horizontalLayout_4.addWidget(self.layersExpandPushButton)
        spacerItem1 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.layersGoUpPushButton = QtWidgets.QPushButton(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersGoUpPushButton.sizePolicy().hasHeightForWidth())
        self.layersGoUpPushButton.setSizePolicy(sizePolicy)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/go-up.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.layersGoUpPushButton.setIcon(icon3)
        self.layersGoUpPushButton.setIconSize(QtCore.QSize(24, 24))
        self.layersGoUpPushButton.setObjectName("layersGoUpPushButton")
        self.horizontalLayout_4.addWidget(self.layersGoUpPushButton)
        self.layersGoDownPushButton = QtWidgets.QPushButton(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersGoDownPushButton.sizePolicy().hasHeightForWidth())
        self.layersGoDownPushButton.setSizePolicy(sizePolicy)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/go-down.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.layersGoDownPushButton.setIcon(icon4)
        self.layersGoDownPushButton.setIconSize(QtCore.QSize(24, 24))
        self.layersGoDownPushButton.setObjectName("layersGoDownPushButton")
        self.horizontalLayout_4.addWidget(self.layersGoDownPushButton)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.layersShapesTreeView = TreeView(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.layersShapesTreeView.sizePolicy().hasHeightForWidth())
        self.layersShapesTreeView.setSizePolicy(sizePolicy)
        self.layersShapesTreeView.setObjectName("layersShapesTreeView")
        self.verticalLayout_5.addWidget(self.layersShapesTreeView)
        self.millSettingsFrame = QtWidgets.QFrame(self.tab_2)
        self.millSettingsFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.millSettingsFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.millSettingsFrame.setLineWidth(0)
        self.millSettingsFrame.setObjectName("millSettingsFrame")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.millSettingsFrame)
        self.verticalLayout_4.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setSpacing(2)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.toolDiameterComboBox = QtWidgets.QComboBox(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolDiameterComboBox.sizePolicy().hasHeightForWidth())
        self.toolDiameterComboBox.setSizePolicy(sizePolicy)
        self.toolDiameterComboBox.setMaxVisibleItems(20)
        self.toolDiameterComboBox.setObjectName("toolDiameterComboBox")
        self.horizontalLayout_3.addWidget(self.toolDiameterComboBox)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_11 = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_11.sizePolicy().hasHeightForWidth())
        self.label_11.setSizePolicy(sizePolicy)
        self.label_11.setText("⌀")
        self.label_11.setTextFormat(QtCore.Qt.PlainText)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout.addWidget(self.label_11)
        self.toolDiameterLabel = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolDiameterLabel.sizePolicy().hasHeightForWidth())
        self.toolDiameterLabel.setSizePolicy(sizePolicy)
        self.toolDiameterLabel.setText("[mm]")
        self.toolDiameterLabel.setObjectName("toolDiameterLabel")
        self.horizontalLayout.addWidget(self.toolDiameterLabel)
        self.label_12 = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_12.sizePolicy().hasHeightForWidth())
        self.label_12.setSizePolicy(sizePolicy)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout.addWidget(self.label_12)
        self.toolSpeedLabel = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolSpeedLabel.sizePolicy().hasHeightForWidth())
        self.toolSpeedLabel.setSizePolicy(sizePolicy)
        self.toolSpeedLabel.setText("[rpm]")
        self.toolSpeedLabel.setObjectName("toolSpeedLabel")
        self.horizontalLayout.addWidget(self.toolSpeedLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_13 = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_13.sizePolicy().hasHeightForWidth())
        self.label_13.setSizePolicy(sizePolicy)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_2.addWidget(self.label_13)
        self.startRadiusLabel = QtWidgets.QLabel(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.startRadiusLabel.sizePolicy().hasHeightForWidth())
        self.startRadiusLabel.setSizePolicy(sizePolicy)
        self.startRadiusLabel.setText("[mm]")
        self.startRadiusLabel.setObjectName("startRadiusLabel")
        self.horizontalLayout_2.addWidget(self.startRadiusLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setHorizontalSpacing(2)
        self.gridLayout.setVerticalSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.zInitialMillDepthLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zInitialMillDepthLineEdit.sizePolicy().hasHeightForWidth())
        self.zInitialMillDepthLineEdit.setSizePolicy(sizePolicy)
        self.zInitialMillDepthLineEdit.setObjectName("zInitialMillDepthLineEdit")
        self.gridLayout.addWidget(self.zInitialMillDepthLineEdit, 4, 1, 1, 1)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_13.setSpacing(2)
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_7 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_7.setWordWrap(True)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_13.addWidget(self.label_7)
        self.unitLabel_8 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_8.setText("[mm/min]")
        self.unitLabel_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_8.setObjectName("unitLabel_8")
        self.horizontalLayout_13.addWidget(self.unitLabel_8)
        self.gridLayout.addLayout(self.horizontalLayout_13, 7, 0, 1, 1)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_12.setSpacing(2)
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_8 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_12.addWidget(self.label_8)
        self.unitLabel_7 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_7.setText("[mm]")
        self.unitLabel_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_7.setObjectName("unitLabel_7")
        self.horizontalLayout_12.addWidget(self.unitLabel_7)
        self.gridLayout.addLayout(self.horizontalLayout_12, 6, 0, 1, 1)
        self.zInfeedDepthLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zInfeedDepthLineEdit.sizePolicy().hasHeightForWidth())
        self.zInfeedDepthLineEdit.setSizePolicy(sizePolicy)
        self.zInfeedDepthLineEdit.setObjectName("zInfeedDepthLineEdit")
        self.gridLayout.addWidget(self.zInfeedDepthLineEdit, 5, 1, 1, 1)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_11.setSpacing(2)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.label_9 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_11.addWidget(self.label_9)
        self.unitLabel_6 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_6.setText("[mm]")
        self.unitLabel_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_6.setObjectName("unitLabel_6")
        self.horizontalLayout_11.addWidget(self.unitLabel_6)
        self.gridLayout.addLayout(self.horizontalLayout_11, 5, 0, 1, 1)
        self.zSafetyMarginLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zSafetyMarginLineEdit.sizePolicy().hasHeightForWidth())
        self.zSafetyMarginLineEdit.setSizePolicy(sizePolicy)
        self.zSafetyMarginLineEdit.setObjectName("zSafetyMarginLineEdit")
        self.gridLayout.addWidget(self.zSafetyMarginLineEdit, 3, 1, 1, 1)
        self.zFinalMillDepthLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zFinalMillDepthLineEdit.sizePolicy().hasHeightForWidth())
        self.zFinalMillDepthLineEdit.setSizePolicy(sizePolicy)
        self.zFinalMillDepthLineEdit.setObjectName("zFinalMillDepthLineEdit")
        self.gridLayout.addWidget(self.zFinalMillDepthLineEdit, 6, 1, 1, 1)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_9.setSpacing(2)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_5 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_9.addWidget(self.label_5)
        self.unitLabel_4 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_4.setText("[mm]")
        self.unitLabel_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_4.setObjectName("unitLabel_4")
        self.horizontalLayout_9.addWidget(self.unitLabel_4)
        self.gridLayout.addLayout(self.horizontalLayout_9, 3, 0, 1, 1)
        self.zRetractionArealLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zRetractionArealLineEdit.sizePolicy().hasHeightForWidth())
        self.zRetractionArealLineEdit.setSizePolicy(sizePolicy)
        self.zRetractionArealLineEdit.setObjectName("zRetractionArealLineEdit")
        self.gridLayout.addWidget(self.zRetractionArealLineEdit, 2, 1, 1, 1)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_8.setSpacing(2)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_6 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_8.addWidget(self.label_6)
        self.unitLabel_3 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_3.setText("[mm]")
        self.unitLabel_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_3.setObjectName("unitLabel_3")
        self.horizontalLayout_8.addWidget(self.unitLabel_3)
        self.gridLayout.addLayout(self.horizontalLayout_8, 2, 0, 1, 1)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_10.setSpacing(2)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_14 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_10.addWidget(self.label_14)
        self.unitLabel_5 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_5.setText("[mm]")
        self.unitLabel_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_5.setObjectName("unitLabel_5")
        self.horizontalLayout_10.addWidget(self.unitLabel_5)
        self.gridLayout.addLayout(self.horizontalLayout_10, 4, 0, 1, 1)
        self.g1FeedZLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.g1FeedZLineEdit.sizePolicy().hasHeightForWidth())
        self.g1FeedZLineEdit.setSizePolicy(sizePolicy)
        self.g1FeedZLineEdit.setObjectName("g1FeedZLineEdit")
        self.gridLayout.addWidget(self.g1FeedZLineEdit, 8, 1, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_6.setSpacing(2)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.startAtXLabel = QtWidgets.QLabel(self.millSettingsFrame)
        self.startAtXLabel.setObjectName("startAtXLabel")
        self.horizontalLayout_6.addWidget(self.startAtXLabel)
        self.unitLabel_1 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_1.setText("[mm]")
        self.unitLabel_1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_1.setObjectName("unitLabel_1")
        self.horizontalLayout_6.addWidget(self.unitLabel_1)
        self.gridLayout.addLayout(self.horizontalLayout_6, 0, 0, 1, 1)
        self.startAtYLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.startAtYLineEdit.sizePolicy().hasHeightForWidth())
        self.startAtYLineEdit.setSizePolicy(sizePolicy)
        self.startAtYLineEdit.setObjectName("startAtYLineEdit")
        self.gridLayout.addWidget(self.startAtYLineEdit, 1, 1, 1, 1)
        self.g1FeedXYLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.g1FeedXYLineEdit.sizePolicy().hasHeightForWidth())
        self.g1FeedXYLineEdit.setSizePolicy(sizePolicy)
        self.g1FeedXYLineEdit.setObjectName("g1FeedXYLineEdit")
        self.gridLayout.addWidget(self.g1FeedXYLineEdit, 7, 1, 1, 1)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_14.setSpacing(2)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_10 = QtWidgets.QLabel(self.millSettingsFrame)
        self.label_10.setWordWrap(True)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_14.addWidget(self.label_10)
        self.unitLabel_9 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_9.setText("[mm/min]")
        self.unitLabel_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_9.setObjectName("unitLabel_9")
        self.horizontalLayout_14.addWidget(self.unitLabel_9)
        self.gridLayout.addLayout(self.horizontalLayout_14, 8, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setContentsMargins(0, -1, 0, -1)
        self.horizontalLayout_7.setSpacing(2)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.startAtYLabel = QtWidgets.QLabel(self.millSettingsFrame)
        self.startAtYLabel.setObjectName("startAtYLabel")
        self.horizontalLayout_7.addWidget(self.startAtYLabel)
        self.unitLabel_2 = QtWidgets.QLabel(self.millSettingsFrame)
        self.unitLabel_2.setText("[mm]")
        self.unitLabel_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.unitLabel_2.setObjectName("unitLabel_2")
        self.horizontalLayout_7.addWidget(self.unitLabel_2)
        self.gridLayout.addLayout(self.horizontalLayout_7, 1, 0, 1, 1)
        self.startAtXLineEdit = QtWidgets.QLineEdit(self.millSettingsFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.startAtXLineEdit.sizePolicy().hasHeightForWidth())
        self.startAtXLineEdit.setSizePolicy(sizePolicy)
        self.startAtXLineEdit.setObjectName("startAtXLineEdit")
        self.gridLayout.addWidget(self.startAtXLineEdit, 0, 1, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout)
        self.verticalLayout_5.addWidget(self.millSettingsFrame)
        self.mytabWidget.addTab(self.tab_2, "")
        self.canvas = Canvas(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(7)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.canvas.sizePolicy().hasHeightForWidth())
        self.canvas.setSizePolicy(sizePolicy)
        self.canvas.setMinimumSize(QtCore.QSize(200, 200))
        self.canvas.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.canvas.setObjectName("canvas")
        self.verticalLayout.addWidget(self.splitter)
        self.messageBox = MessageBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.messageBox.sizePolicy().hasHeightForWidth())
        self.messageBox.setSizePolicy(sizePolicy)
        self.messageBox.setMaximumSize(QtCore.QSize(16777215, 100))
        self.messageBox.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.messageBox.setObjectName("messageBox")
        self.verticalLayout.addWidget(self.messageBox)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 24))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuExport = QtWidgets.QMenu(self.menubar)
        self.menuExport.setEnabled(True)
        self.menuExport.setObjectName("menuExport")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        self.menuOptions = QtWidgets.QMenu(self.menubar)
        self.menuOptions.setObjectName("menuOptions")
        self.menuMachine_Type = QtWidgets.QMenu(self.menuOptions)
        self.menuMachine_Type.setObjectName("menuMachine_Type")
        self.menu_Measurement_units = QtWidgets.QMenu(self.menuOptions)
        self.menu_Measurement_units.setEnabled(False)
        self.menu_Measurement_units.setToolTip("")
        self.menu_Measurement_units.setObjectName("menu_Measurement_units")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionOpen = QtWidgets.QAction(MainWindow)
        self.actionOpen.setObjectName("actionOpen")
        self.actionClose = QtWidgets.QAction(MainWindow)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/delete.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionClose.setIcon(icon5)
        self.actionClose.setObjectName("actionClose")
        self.actionShowPathDirections = QtWidgets.QAction(MainWindow)
        self.actionShowPathDirections.setCheckable(True)
        self.actionShowPathDirections.setChecked(False)
        self.actionShowPathDirections.setEnabled(False)
        self.actionShowPathDirections.setObjectName("actionShowPathDirections")
        self.actionShowDisabledPaths = QtWidgets.QAction(MainWindow)
        self.actionShowDisabledPaths.setCheckable(True)
        self.actionShowDisabledPaths.setChecked(False)
        self.actionShowDisabledPaths.setEnabled(False)
        self.actionShowDisabledPaths.setObjectName("actionShowDisabledPaths")
        self.actionAutoscale = QtWidgets.QAction(MainWindow)
        self.actionAutoscale.setEnabled(False)
        self.actionAutoscale.setObjectName("actionAutoscale")
        self.actionDeleteG0Paths = QtWidgets.QAction(MainWindow)
        self.actionDeleteG0Paths.setEnabled(False)
        self.actionDeleteG0Paths.setObjectName("actionDeleteG0Paths")
        self.actionConfiguration = QtWidgets.QAction(MainWindow)
        self.actionConfiguration.setObjectName("actionConfiguration")
        self.actionConfigurationPostprocessor = QtWidgets.QAction(MainWindow)
        self.actionConfigurationPostprocessor.setObjectName("actionConfigurationPostprocessor")
        self.actionTolerances = QtWidgets.QAction(MainWindow)
        self.actionTolerances.setObjectName("actionTolerances")
        self.actionScaleAll = QtWidgets.QAction(MainWindow)
        self.actionScaleAll.setEnabled(False)
        self.actionScaleAll.setObjectName("actionScaleAll")
        self.actionRotateAll = QtWidgets.QAction(MainWindow)
        self.actionRotateAll.setEnabled(False)
        self.actionRotateAll.setObjectName("actionRotateAll")
        self.actionMoveWorkpieceZero = QtWidgets.QAction(MainWindow)
        self.actionMoveWorkpieceZero.setEnabled(False)
        self.actionMoveWorkpieceZero.setObjectName("actionMoveWorkpieceZero")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionOptimizePaths = QtWidgets.QAction(MainWindow)
        self.actionOptimizePaths.setEnabled(False)
        self.actionOptimizePaths.setObjectName("actionOptimizePaths")
        self.actionExportShapes = QtWidgets.QAction(MainWindow)
        self.actionExportShapes.setEnabled(False)
        self.actionExportShapes.setObjectName("actionExportShapes")
        self.actionOptimizeAndExportShapes = QtWidgets.QAction(MainWindow)
        self.actionOptimizeAndExportShapes.setEnabled(False)
        self.actionOptimizeAndExportShapes.setObjectName("actionOptimizeAndExportShapes")
        self.actionLiveUpdateExportRoute = QtWidgets.QAction(MainWindow)
        self.actionLiveUpdateExportRoute.setCheckable(True)
        self.actionLiveUpdateExportRoute.setEnabled(False)
        self.actionLiveUpdateExportRoute.setObjectName("actionLiveUpdateExportRoute")
        self.actionReload = QtWidgets.QAction(MainWindow)
        self.actionReload.setEnabled(False)
        self.actionReload.setObjectName("actionReload")
        self.actionSplitLineSegments = QtWidgets.QAction(MainWindow)
        self.actionSplitLineSegments.setCheckable(True)
        self.actionSplitLineSegments.setObjectName("actionSplitLineSegments")
        self.actionAutomaticCutterCompensation = QtWidgets.QAction(MainWindow)
        self.actionAutomaticCutterCompensation.setCheckable(True)
        self.actionAutomaticCutterCompensation.setEnabled(False)
        self.actionAutomaticCutterCompensation.setObjectName("actionAutomaticCutterCompensation")
        self.actionMilling = QtWidgets.QAction(MainWindow)
        self.actionMilling.setCheckable(True)
        self.actionMilling.setObjectName("actionMilling")
        self.actionDragKnife = QtWidgets.QAction(MainWindow)
        self.actionDragKnife.setCheckable(True)
        self.actionDragKnife.setObjectName("actionDragKnife")
        self.actionLathe = QtWidgets.QAction(MainWindow)
        self.actionLathe.setCheckable(True)
        self.actionLathe.setObjectName("actionLathe")
        self.actionTopView = QtWidgets.QAction(MainWindow)
        self.actionTopView.setEnabled(False)
        self.actionTopView.setObjectName("actionTopView")
        self.actionIsometricView = QtWidgets.QAction(MainWindow)
        self.actionIsometricView.setEnabled(False)
        self.actionIsometricView.setObjectName("actionIsometricView")
        self.actionSaveProjectAs = QtWidgets.QAction(MainWindow)
        self.actionSaveProjectAs.setEnabled(False)
        self.actionSaveProjectAs.setObjectName("actionSaveProjectAs")
        self.actionMillimeters = QtWidgets.QAction(MainWindow)
        self.actionMillimeters.setCheckable(True)
        self.actionMillimeters.setEnabled(True)
        self.actionMillimeters.setObjectName("actionMillimeters")
        self.actionInches = QtWidgets.QAction(MainWindow)
        self.actionInches.setCheckable(True)
        self.actionInches.setObjectName("actionInches")
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionReload)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSaveProjectAs)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionClose)
        self.menuExport.addAction(self.actionOptimizePaths)
        self.menuExport.addAction(self.actionExportShapes)
        self.menuExport.addAction(self.actionOptimizeAndExportShapes)
        self.menuView.addAction(self.actionShowPathDirections)
        self.menuView.addAction(self.actionShowDisabledPaths)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionLiveUpdateExportRoute)
        self.menuView.addAction(self.actionDeleteG0Paths)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionAutoscale)
        self.menuView.addAction(self.actionTopView)
        self.menuView.addAction(self.actionIsometricView)
        self.menuMachine_Type.addAction(self.actionMilling)
        self.menuMachine_Type.addAction(self.actionLathe)
        self.menuMachine_Type.addAction(self.actionDragKnife)
        self.menu_Measurement_units.addAction(self.actionMillimeters)
        self.menu_Measurement_units.addAction(self.actionInches)
        self.menuOptions.addAction(self.actionConfiguration)
        self.menuOptions.addAction(self.actionConfigurationPostprocessor)
        self.menuOptions.addSeparator()
        self.menuOptions.addAction(self.actionTolerances)
        self.menuOptions.addSeparator()
        self.menuOptions.addAction(self.actionScaleAll)
        self.menuOptions.addAction(self.actionRotateAll)
        self.menuOptions.addSeparator()
        self.menuOptions.addAction(self.actionMoveWorkpieceZero)
        self.menuOptions.addSeparator()
        self.menuOptions.addAction(self.actionSplitLineSegments)
        self.menuOptions.addAction(self.actionAutomaticCutterCompensation)
        self.menuOptions.addAction(self.menuMachine_Type.menuAction())
        self.menuOptions.addAction(self.menu_Measurement_units.menuAction())
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuExport.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuOptions.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.mytabWidget.setCurrentIndex(1)
        self.layersCollapsePushButton.clicked.connect(self.layersShapesTreeView.collapseAll) # type: ignore
        self.layersExpandPushButton.clicked.connect(self.layersShapesTreeView.expandAll) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.mytabWidget, self.entitiesTreeView)
        MainWindow.setTabOrder(self.entitiesTreeView, self.blocksCollapsePushButton)
        MainWindow.setTabOrder(self.blocksCollapsePushButton, self.blocksExpandPushButton)
        MainWindow.setTabOrder(self.blocksExpandPushButton, self.canvas)
        MainWindow.setTabOrder(self.canvas, self.messageBox)
        MainWindow.setTabOrder(self.messageBox, self.layersShapesTreeView)
        MainWindow.setTabOrder(self.layersShapesTreeView, self.layersCollapsePushButton)
        MainWindow.setTabOrder(self.layersCollapsePushButton, self.layersExpandPushButton)
        MainWindow.setTabOrder(self.layersExpandPushButton, self.layersGoUpPushButton)
        MainWindow.setTabOrder(self.layersGoUpPushButton, self.layersGoDownPushButton)
        MainWindow.setTabOrder(self.layersGoDownPushButton, self.toolDiameterComboBox)
        MainWindow.setTabOrder(self.toolDiameterComboBox, self.startAtXLineEdit)
        MainWindow.setTabOrder(self.startAtXLineEdit, self.startAtYLineEdit)
        MainWindow.setTabOrder(self.startAtYLineEdit, self.zRetractionArealLineEdit)
        MainWindow.setTabOrder(self.zRetractionArealLineEdit, self.zSafetyMarginLineEdit)
        MainWindow.setTabOrder(self.zSafetyMarginLineEdit, self.zInitialMillDepthLineEdit)
        MainWindow.setTabOrder(self.zInitialMillDepthLineEdit, self.zInfeedDepthLineEdit)
        MainWindow.setTabOrder(self.zInfeedDepthLineEdit, self.zFinalMillDepthLineEdit)
        MainWindow.setTabOrder(self.zFinalMillDepthLineEdit, self.g1FeedXYLineEdit)
        MainWindow.setTabOrder(self.g1FeedXYLineEdit, self.g1FeedZLineEdit)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "DXF2GCODE"))
        self.blocksCollapsePushButton.setToolTip(_translate("MainWindow", "Collapse all items"))
        self.blocksExpandPushButton.setToolTip(_translate("MainWindow", "Expand all items"))
        self.mytabWidget.setTabText(self.mytabWidget.indexOf(self.tab), _translate("MainWindow", "Entities"))
        self.layersCollapsePushButton.setToolTip(_translate("MainWindow", "Collapse all items"))
        self.layersExpandPushButton.setToolTip(_translate("MainWindow", "Expand all items"))
        self.layersGoUpPushButton.setToolTip(_translate("MainWindow", "Move-up the selected shape/layer"))
        self.layersGoDownPushButton.setToolTip(_translate("MainWindow", "Move-down the selected shape/layer"))
        self.label_12.setText(_translate("MainWindow", "/ speed "))
        self.label_13.setText(_translate("MainWindow", "start rad. (comp) "))
        self.zInitialMillDepthLineEdit.setToolTip(_translate("MainWindow", "Milling will start at Z = \"Workpiece top Z\" - \"Z infeed depth\""))
        self.label_7.setText(_translate("MainWindow", "Feed rate XY"))
        self.label_8.setText(_translate("MainWindow", "Z Final mill depth"))
        self.label_9.setText(_translate("MainWindow", "Z Infeed depth"))
        self.label_5.setText(_translate("MainWindow", "Z Safety margin"))
        self.label_6.setText(_translate("MainWindow", "Z Retraction area"))
        self.label_14.setText(_translate("MainWindow", "Z Workpiece top"))
        self.startAtXLabel.setText(_translate("MainWindow", "Start X"))
        self.label_10.setText(_translate("MainWindow", "Feed rate Z"))
        self.startAtYLabel.setText(_translate("MainWindow", "Start Y"))
        self.mytabWidget.setTabText(self.mytabWidget.indexOf(self.tab_2), _translate("MainWindow", "Layers"))
        self.menuFile.setTitle(_translate("MainWindow", "&File"))
        self.menuExport.setStatusTip(_translate("MainWindow", "Export the current project to G-Code"))
        self.menuExport.setTitle(_translate("MainWindow", "&Export"))
        self.menuView.setTitle(_translate("MainWindow", "&View"))
        self.menuOptions.setTitle(_translate("MainWindow", "&Options"))
        self.menuMachine_Type.setTitle(_translate("MainWindow", "Mac&hine Type"))
        self.menu_Measurement_units.setTitle(_translate("MainWindow", "Measurement &units"))
        self.menuHelp.setTitle(_translate("MainWindow", "&Help"))
        self.actionOpen.setText(_translate("MainWindow", "&Open..."))
        self.actionOpen.setStatusTip(_translate("MainWindow", "Load DXF or other supported document"))
        self.actionOpen.setShortcut(_translate("MainWindow", "Ctrl+O"))
        self.actionClose.setText(_translate("MainWindow", "&Exit"))
        self.actionClose.setStatusTip(_translate("MainWindow", "Exit DXF2GCODE and close window"))
        self.actionClose.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.actionShowPathDirections.setText(_translate("MainWindow", "&Show Path Directions"))
        self.actionShowPathDirections.setStatusTip(_translate("MainWindow", "Always shows the path direction in the plot (not only while selected)"))
        self.actionShowDisabledPaths.setText(_translate("MainWindow", "Show Disabled &Paths"))
        self.actionAutoscale.setText(_translate("MainWindow", "&Autoscale"))
        self.actionAutoscale.setShortcut(_translate("MainWindow", "Ctrl+V, A"))
        self.actionDeleteG0Paths.setText(_translate("MainWindow", "&Delete G0 Paths"))
        self.actionConfiguration.setText(_translate("MainWindow", "&Configuration..."))
        self.actionConfiguration.setShortcut(_translate("MainWindow", "Ctrl+Shift+C"))
        self.actionConfigurationPostprocessor.setText(_translate("MainWindow", "&Postprocessor configuration..."))
        self.actionConfigurationPostprocessor.setShortcut(_translate("MainWindow", "Ctrl+Shift+P"))
        self.actionTolerances.setText(_translate("MainWindow", "&Tolerances"))
        self.actionScaleAll.setText(_translate("MainWindow", "&Scale All"))
        self.actionRotateAll.setText(_translate("MainWindow", "&Rotate All"))
        self.actionMoveWorkpieceZero.setText(_translate("MainWindow", "&Move Workpiece Zero"))
        self.actionAbout.setText(_translate("MainWindow", "&About"))
        self.actionOptimizePaths.setText(_translate("MainWindow", "&Optimize Paths"))
        self.actionOptimizePaths.setShortcut(_translate("MainWindow", "Ctrl+Shift+O"))
        self.actionExportShapes.setText(_translate("MainWindow", "&Export Shapes"))
        self.actionExportShapes.setShortcut(_translate("MainWindow", "Ctrl+Shift+E"))
        self.actionOptimizeAndExportShapes.setText(_translate("MainWindow", "Optimize &and Export Shapes"))
        self.actionOptimizeAndExportShapes.setShortcut(_translate("MainWindow", "Ctrl+E"))
        self.actionLiveUpdateExportRoute.setText(_translate("MainWindow", "&Live Update Export Route"))
        self.actionReload.setText(_translate("MainWindow", "&Reload"))
        self.actionReload.setShortcut(_translate("MainWindow", "Ctrl+R"))
        self.actionSplitLineSegments.setText(_translate("MainWindow", "Split &Line Segments"))
        self.actionSplitLineSegments.setStatusTip(_translate("MainWindow", "Split line segments, e.g. can be used for compensation (G41/G42) in combination with EMC"))
        self.actionAutomaticCutterCompensation.setText(_translate("MainWindow", "&Automatic Cutter Compensation"))
        self.actionMilling.setText(_translate("MainWindow", "&Milling"))
        self.actionDragKnife.setText(_translate("MainWindow", "&Drag Knife"))
        self.actionLathe.setText(_translate("MainWindow", "&Lathe"))
        self.actionTopView.setText(_translate("MainWindow", "&Top View"))
        self.actionTopView.setShortcut(_translate("MainWindow", "Ctrl+V, T"))
        self.actionIsometricView.setText(_translate("MainWindow", "&Isometric View"))
        self.actionIsometricView.setShortcut(_translate("MainWindow", "Ctrl+V, I"))
        self.actionSaveProjectAs.setText(_translate("MainWindow", "&Save Project As..."))
        self.actionSaveProjectAs.setShortcut(_translate("MainWindow", "Ctrl+S"))
        self.actionMillimeters.setText(_translate("MainWindow", "&Millimeters"))
        self.actionInches.setText(_translate("MainWindow", "&Inches"))
from dxf2gcode.gui.canvas import Canvas
from dxf2gcode.gui.messagebox import MessageBox
from dxf2gcode.gui.treeview import TreeView
import dxf2gcode_images5_rc



# -*- coding: utf-8 -*-

############################################################################
#
#   Copyright (C) 2008-2015
#    Christian Kohlöffel
#    Vinzenz Schulz
#    Jean-Paul Schouwstra
#
#   This file is part of DXF2GCODE.
#
#   DXF2GCODE is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   DXF2GCODE is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with DXF2GCODE.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################

from __future__ import absolute_import
from __future__ import division

from math import sin, cos, acos, pi
import logging

from dxf2gcode.core.point import Point

import dxf2gcode.globals.constants as c
from PyQt5.QtWidgets import QGraphicsLineItem, QGraphicsItem
from PyQt5.QtGui import QPen, QPolygonF
from PyQt5 import QtCore


logger = logging.getLogger("Gui.Arrow")


class Arrow(QGraphicsLineItem):
    def __init__(self, startp=Point(), endp=None,
                 length=60.0, angle=50.0,
                 color=QtCore.Qt.red, pencolor=QtCore.Qt.green,
                 startarrow=True):
        """
        Initialisation of the class.
        """
        self.sc = None
        super(Arrow, self).__init__()

        self.startp = QtCore.QPointF(startp.x, -startp.y)
        self.endp = endp

        self.length = length
        self.angle = angle
        self.startarrow = startarrow
        self.allwaysshow = False

        self.arrowHead = QPolygonF()
        self.setFlag(QGraphicsItem.ItemIsSelectable, False)
        self.myColor = color
        self.pen = QPen(pencolor, 1, QtCore.Qt.SolidLine)
        self.pen.setCosmetic(True)
        self.arrowSize = 8.0

    def contains_point(self, point):
        """
        Arrows cannot be selected. Return maximal distance
        """
        return float(0x7fffffff)

    def setSelected(self, flag=True, blockSignals=True):
        """
        Override inherited function to turn off selection of Arrows.
        @param flag: The flag to enable or disable Selection
        """
        if self.allwaysshow:
            pass
        elif flag is True:
            self.show()
        else:
            self.hide()

    def setallwaysshow(self, flag=False):
        """
        If the directions shall be allwaysshown the parameter
        will be set and  all paths will be shown.
        @param flag: The flag to enable or disable Selection
        """
        self.allwaysshow = flag
        if flag is True:
            self.show()
        elif flag is True and self.isSelected():
            self.show()
        else:
            self.hide()

    def paint(self, painter, option, widget=None):
        """
        Method for painting the arrow.
        """

        demat = painter.deviceTransform()
        self.sc = demat.m11()

        if self.endp is None:
            dx = cos(self.angle) * self.length / self.sc
            dy = sin(self.angle) * self.length / self.sc

            endp = QtCore.QPointF(self.startp.x() - dx, self.startp.y() + dy)
        else:
            endp = QtCore.QPointF(self.endp.x, -self.endp.y)

        arrowSize = self.arrowSize / self.sc

        painter.setPen(self.pen)
        painter.setBrush(self.myColor)

        self.setLine(QtCore.QLineF(endp, self.startp))
        line = self.line()

        if line.length() != 0:
            angle = acos(line.dx() / line.length())

            if line.dy() >= 0:
                angle = (pi * 2.0) - angle

            if self.startarrow:
                arrowP1 = line.p2() - QtCore.QPointF(sin(angle + pi / 3.0) * arrowSize,
                                                     cos(angle + pi / 3.0) * arrowSize)
                arrowP2 = line.p2() - QtCore.QPointF(sin(angle + pi - pi / 3.0) * arrowSize,
                                                     cos(angle + pi - pi / 3.0) * arrowSize)
                self.arrowHead.clear()
                for Point in [line.p2(), arrowP1, arrowP2]:
                    self.arrowHead.append(Point)

            else:
                arrowP1 = line.p1() + QtCore.QPointF(sin(angle + pi / 3.0) * arrowSize,
                                                     cos(angle + pi / 3.0) * arrowSize)
                arrowP2 = line.p1() + QtCore.QPointF(sin(angle + pi - pi / 3.0) * arrowSize,
                                                     cos(angle + pi - pi / 3.0) * arrowSize)
                self.arrowHead.clear()
                for Point in [line.p1(), arrowP1, arrowP2]:
                    self.arrowHead.append(Point)

            painter.drawLine(line)
            painter.drawPolygon(self.arrowHead)

    def boundingRect(self):
        """
        Override inherited function to enlarge selection of Arrow to include all
        @param flag: The flag to enable or disable Selection
        """
        if not self.sc:  # since this function is called before paint; and scale is unknown
            return QtCore.QRectF(self.startp.x(), self.startp.y(), 1e-9, 1e-9)

        arrowSize = self.arrowSize / self.sc
        extra = arrowSize  # self.pen.width() +

        if self.endp is None:
            dx = cos(self.angle) * self.length / self.sc
            dy = sin(self.angle) * self.length / self.sc

            endp = QtCore.QPointF(self.startp.x() - dx, self.startp.y() + dy)
        else:
            endp = QtCore.QPointF(self.endp.x, -self.endp.y)

        brect = QtCore.QRectF(self.startp,
                              QtCore.QSizeF(endp.x()-self.startp.x(),
                                            endp.y()-self.startp.y())).normalized().adjusted(-extra, -extra, extra, extra)
        return brect



# -*- coding: utf-8 -*-

############################################################################
#
#   Copyright (C) 2008-2015
#    Christian Kohlöffel
#    Vinzenz Schulz
#    Jean-Paul Schouwstra
#
#   This file is part of DXF2GCODE.
#
#   DXF2GCODE is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   DXF2GCODE is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with DXF2GCODE.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################

from __future__ import absolute_import
from __future__ import division

from random import random, shuffle
from math import floor, ceil

import dxf2gcode.globals.globals as g

import dxf2gcode.globals.constants as c
from PyQt5 import QtCore

import logging
logger = logging.getLogger("PostPro.TSP")


class TspOptimization(object):
    """
    Optimization using the Travelling Salesman Problem (TSP) algorithim
    """
    def __init__(self, st_end_points, order):
        self.shape_nrs = len(st_end_points)
        self.iterations = int(self.shape_nrs) * 10
        self.pop_nr = min(int(ceil(self.shape_nrs / 8.0) * 8.0),
                          g.config.vars.Route_Optimisation['max_population'])
        self.mutate_rate = g.config.vars.Route_Optimisation['mutation_rate']
        self.opt_route = []
        self.order = order
        self.st_end_points = st_end_points

        # Generate the Distance Matrix
        self.DistanceMatrix = DistanceMatrixClass()
        self.DistanceMatrix.generate_matrix(st_end_points)

        # Generation Population
        self.Population = PopulationClass([self.shape_nrs, self.pop_nr],
                                          self.DistanceMatrix.matrix,
                                          self.mutate_rate)

        # Initialise the Result Class
        self.Fittness = FittnessClass(self.Population,
                                      list(range(self.Population.size[1])),
                                      self.order)
        self.Fittness.calc_st_fittness(self.DistanceMatrix.matrix,
                                       range(self.shape_nrs))

        # Anfang der Reihenfolge immer auf den letzen Punkt legen
        # Beginning of the sequence always put the last point ???
        self.Fittness.set_startpoint()

        # Function to correct the order of the elements
        self.Fittness.correct_constrain_order()

        # logger.debug('Calculation of start fitness TSP: %s' %self)
        # logger.debug('Size Distance matrix: %s', len(self.DistanceMatrix.matrix))
        # Erstellen der ersten Ergebnisse
        # Create the first result
        self.Fittness.calc_cur_fittness(self.DistanceMatrix.matrix)
        self.Fittness.select_best_fittness()
        self.opt_route = self.Population.pop[self.Fittness.best_route]

        # ERstellen der 2 opt Optimierungs Klasse
        # Create the 2 opt optimization class ???
        # self.optmove=ClassOptMove(dmatrix=self.DistanceMatrix.matrix, nei_nr=int(round(self.shape_nrs/10)))

    def calc_next_iteration(self):
        """
        calc_next_iteration()
        """
        # Algorithmus ausfürhen
        self.Population.genetic_algorithm(self.Fittness, self.mutate_rate)
        # Für die Anzahl der Tours die Tours nach dem 2-opt Verfahren optimieren
        # Optimise the number of Tours de Tours to the 2-opt method ???
#        for pop_nr in range(len(self.Population.pop)):
#            #print ("Vorher: %0.2f" %self.calc_tour_length(tours[tour_nr]))
#            self.Population.pop[pop_nr]=self.optmove.do2optmove(self.Population.pop[pop_nr])
#            #print ("Nachher: %0.2f" %self.calc_tour_length(tours[tour_nr]))
        # Anfang der Reihenfolge immer auf den letzen Punkt legen
        # Always put the last point at the beginning of the sequence
        self.Fittness.set_startpoint()
        # Korrektur Funktion um die Reihenfolge der Elemente zu korrigieren
        # Function to correct the order of the elements
        self.Fittness.correct_constrain_order()
        # Fittness der jeweiligen Routen ausrechen
        # Calculate fitness of each route
        self.Fittness.calc_cur_fittness(self.DistanceMatrix.matrix)
        # Straffunktion falls die Route nicht der gewünschten Reihenfolge entspricht
        # Function if the route is not the desired sequence ???
        # Best route to choose
        self.Fittness.select_best_fittness()
        self.opt_route = self.Population.pop[self.Fittness.best_route]
        # logger.debug('Calculation next iteration of TSP: %s' %self)

    def __str__(self):
        #res = self.Population.pop
        return "Iteration nrs:    %i" % (self.iterations * 10) +\
               "\nShape nrs:      %i" % self.shape_nrs +\
               "\nPopulation:     %i" % self.pop_nr +\
               "\nMutate rate:    %0.2f" % self.mutate_rate +\
               "\norder:          %s" % self.order +\
               "\nStart length:   %0.1f" % self.Fittness.best_fittness[0] +\
               "\nOpt. length:    %0.1f" % self.Fittness.best_fittness[-1] +\
               "\nOpt. route:     %s" % self.opt_route

class PopulationClass:
    def __init__(self, size, dmatrix, mutate_rate):
        self.size = size
        self.mutate_rate = mutate_rate
        self.pop = []
        self.rot = []

        # logger.debug('The Population size is: %s' %self.size)

        for pop_nr in range(self.size[1]):
            # logger.debug("======= TSP initializing population nr %i =======" % pop_nr)

            if g.config.vars.Route_Optimisation['begin_art'] == 'ordered':
                self.pop.append(list(range(size[0])))
            elif g.config.vars.Route_Optimisation['begin_art'] == 'random':
                self.pop.append(self.random_begin(size[0]))
            elif g.config.vars.Route_Optimisation['begin_art'] == 'heuristic':
                self.pop.append(self.heuristic_begin(dmatrix[:]))
            else:
                logger.error(self.tr('Wrong begin art of TSP chosen'))

        for rot_nr in range(size[0]):
            self.rot.append(0)

    def __str__(self):
        string = "\nPopulation size: %i X %i \nMutate rate: %0.2f \nRotation Matrix:\n%s \nPop Matrix:"\
                 % (self.size[0], self.size[1], self.mutate_rate, self.rot)

        for line in self.pop:
            string += '\n' + str(line)
        return string

    def tr(self, string_to_translate):
        """
        Translate a string using the QCoreApplication translation framework
        @param: string_to_translate: a unicode string
        @return: the translated unicode string if it was possible to translate
        """
        return str(QtCore.QCoreApplication.translate("PopulationClass",
                                                           string_to_translate))

    def random_begin(self, size):
        """
        random_begin for TSP
        """
        tour = list(range(size))
        shuffle(tour)
        return tour

    def heuristic_begin(self, dmatrix):
        """
        heuristic_begin for TSP
        """
        tour = []
        possibilities = list(range(len(dmatrix[0])))
        start_nr = int(floor(random()*len(dmatrix[0])))

        # Hinzufügen der Nr und entfernen aus possibilies
        # Add and remove the number of possibilities
        tour.append(start_nr)
        possibilities.pop(possibilities.index(tour[-1]))
        counter = 0

        while len(possibilities):
            counter += 1
            tour.append(self.heuristic_find_next(tour[-1], possibilities, dmatrix))
            possibilities.pop(possibilities.index(tour[-1]))

            # if counter % 10 == 0:
            #     logger.debug("TSP heuristic searching nr %i" % counter)
        return tour

    def heuristic_find_next(self, start, possibilities, dmatrix):
        """
        heuristic_find_next() for TSP
        """
        # Auswahl der Entfernungen des nächsten Punkts
        # The distances of the point selection
        min_dist = 1e99
        darray = dmatrix[start]

        for pnr in possibilities:
            if darray[pnr] < min_dist:
                min_point = pnr
                min_dist = darray[pnr]
        return min_point

    def genetic_algorithm(self, Result, mutate_rate):
        """
        genetic_algorithm for TSP
        """
        self.mutate_rate = mutate_rate

        # Neue Population Matrix erstellen
        # Create new Population Matrix
        new_pop = []
        for p_nr in range(self.size[1]):
            new_pop.append([])

        # Tournament Selection 1 between Parents (2 Parents remaining)
        ts_r1 = list(range(self.size[1]))
        shuffle(ts_r1)
        winners_r1 = []
        tmp_fittness = []
        for nr in range(self.size[1] // 2):
            if Result.cur_fittness[ts_r1[nr * 2]] < Result.cur_fittness[ts_r1[(nr * 2) + 1]]:
                winners_r1.append(self.pop[ts_r1[nr * 2]])
                tmp_fittness.append(Result.cur_fittness[ts_r1[nr * 2]])
            else:
                winners_r1.append(self.pop[ts_r1[(nr * 2) + 1]])
                tmp_fittness.append(Result.cur_fittness[ts_r1[(nr * 2) + 1]])
        # print(tmp_fittness)

        # Tournament Selection 2 only one Parent remaining
        ts_r2 = list(range(self.size[1] // 2))
        shuffle(ts_r2)
        for nr in range(self.size[1] // 4):
            if tmp_fittness[ts_r2[nr * 2]] < tmp_fittness[ts_r2[(nr * 2) + 1]]:
                winner = winners_r1[ts_r2[nr * 2]]
            else:
                winner = winners_r1[ts_r2[(nr * 2) + 1]]

            # Schreiben der Gewinner in die neue Population Matrix
            # print(winner)
            for pnr in range(2):
                new_pop[pnr * self.size[1] // 2 + nr] = winner[:]

        # Crossover Gens from 2 Parents
        crossover = list(range(self.size[1] // 2))
        shuffle(crossover)
        for nr in range(self.size[1] // 4):
            # child = parent2
            # Parents are the winners of the first round (Genetic Selection?)
            parent1 = winners_r1[crossover[nr * 2]][:]
            child = winners_r1[crossover[(nr * 2) + 1]][:]

            # The genetic line that is exchanged in the child parent1
            indx = [int(floor(random()*self.size[0])), int(floor(random()*self.size[0]))]
            indx.sort()
            while indx[0] == indx[1]:
                indx = [int(floor(random()*self.size[0])), int(floor(random()*self.size[0]))]
                indx.sort()
            gens = parent1[indx[0]:indx[1] + 1]

            # Remove the exchanged genes
            for gen in gens:
                child.pop(child.index(gen))

            # Insert the new genes at a random position
            ins_indx = int(floor(random()*self.size[0]))
            new_children = child[0:ins_indx] + gens + child[ins_indx:len(child)]

            # Write the new children in the new population matrix
            for pnr in range(2):
                new_pop[int((pnr + 0.5) * self.size[1] / 2 + nr)] = new_children[:]

        # Mutate the 2nd half of the population matrix
        mutate = list(range(self.size[1] // 2))
        shuffle(mutate)
        num_mutations = int(round(mutate_rate * self.size[1] / 2))
        for nr in range(num_mutations):
            # The genetic line that is exchanged in the child parent1 ???
            indx = [int(floor(random()*self.size[0])), int(floor(random()*self.size[0]))]
            indx.sort()
            while indx[0] == indx[1]:
                indx = [int(floor(random()*self.size[0])), int(floor(random()*self.size[0]))]
                indx.sort()

            # Zu mutierende Line
            # Line to be mutated ????
            mutline = new_pop[self.size[1] // 2 + mutate[nr]]
            if random() < 0.75:  # Gen Abschnitt umdrehen / Turn gene segment
                cut = mutline[indx[0]:indx[1] + 1]
                cut.reverse()
                mutline = mutline[0:indx[0]] + cut + mutline[indx[1] + 1:len(mutline)]
            else:  # 2 Gene tauschen / 2 Gene exchange
                orgline = mutline[:]
                mutline[indx[0]] = orgline[indx[1]]
                mutline[indx[1]] = orgline[indx[0]]
            new_pop[self.size[1] // 2 + mutate[nr]] = mutline

        # Assign the new population matrix
        self.pop = new_pop

class DistanceMatrixClass:
    """
    DistanceMatrixClass
    """
    def __init__(self):
        self.matrix = []
        self.size = [0, 0]

    def __str__(self):
        string = ("Distance Matrix; size: %i X %i" % (self.size[0], self.size[1]))
        for line_x in self.matrix:
            string += "\n"
            for x_vals in line_x:
                string += "%8.2f" % x_vals
        return string

    def generate_matrix(self, st_end_points):
        self.matrix = [[st_end_y[1].distance(st_end_x[0]) for st_end_x in st_end_points]
                       for st_end_y in st_end_points]
        self.size = [len(st_end_points), len(st_end_points)]

class FittnessClass:
    def __init__(self, population, cur_fittness, order):
        self.population = population
        self.cur_fittness = cur_fittness
        self.order = order
        self.best_fittness = []
        self.best_route = []

    def __str__(self):
        return "\nBest Fittness: %s \nBest Route: %s \nBest Pop: %s"\
               % (self.best_fittness[-1], self.best_route, self.population.pop[self.best_route])

    def calc_st_fittness(self, matrix, st_pop):
        dis = matrix[st_pop[-1]][st_pop[0]]
        for nr in range(1, len(st_pop)):
            dis += matrix[st_pop[nr - 1]][st_pop[nr]]

        self.best_fittness.append(dis)

    def calc_cur_fittness(self, matrix):
        # logger.debug("Calculating current fittness len(self.population.pop): %s"
        #              % len(self.population.pop))
        # logger.debug("Length of self.cur_fittness: %s" %(len(self.cur_fittness)))

        for pop_nr in range(len(self.population.pop)):
            pop = self.population.pop[pop_nr]
            # logger.debug("pop_nr: %s" %pop_nr)
            dis = matrix[pop[-1]][pop[0]]
            for nr in range(1, len(pop)):
                dis += matrix[pop[nr - 1]][pop[nr]]
            self.cur_fittness[pop_nr] = dis

    # 2te Möglichkeit die Reihenfolge festzulegen (Korrekturfunktion=Aktiv)
    # Second option set the order (correction function = Active)
    def correct_constrain_order(self):
        """FIXME: in order to change the correction to have all ordered shapes
        in begin this might be the best place to change it. Maybe we can also have
        an additional option in the config file?"""

        for pop in self.population.pop:
            # Search the current order
            order_index = self.get_pop_index_list(pop)
            # Momentane Reihenfolge der indexe sortieren
            # Current sort order of the index ???
            order_index.sort()
            # Indices according to correct order
            for ind_nr in range(len(order_index)):
                pop[order_index[ind_nr]] = self.order[ind_nr]

    def set_startpoint(self):
        n_pts = len(self.population.pop[-1])
        for pop in self.population.pop:
            st_pt_nr = pop.index(n_pts - 1)
            # 99% of the time we don't have to reorder the arrays
            if st_pt_nr > 0:
                # Contour with the starting point at the beginning
                pop[:] = pop[st_pt_nr:n_pts] + pop[0:st_pt_nr]

    def get_pop_index_list(self, pop):
        return [pop.index(order) for order in self.order]

    def select_best_fittness(self):
        self.best_fittness.append(min(self.cur_fittness))
        self.best_route = self.cur_fittness.index(self.best_fittness[-1])



# -*- coding: utf-8 -*-

############################################################################
#
#   Copyright (C) 2008-2015
#    Christian Kohlöffel
#    Vinzenz Schulz
#    Jean-Paul Schouwstra
#
#   This file is part of DXF2GCODE.
#
#   DXF2GCODE is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   DXF2GCODE is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with DXF2GCODE.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################

from __future__ import absolute_import
from __future__ import division

from math import radians, pi
from copy import deepcopy
import logging

import dxf2gcode.globals.globals as g
from dxf2gcode.core.point import Point
from dxf2gcode.core.linegeo import LineGeo
from dxf2gcode.core.arcgeo import ArcGeo
from dxf2gcode.core.holegeo import HoleGeo

import dxf2gcode.globals.constants as c
from PyQt5 import QtCore

logger = logging.getLogger("Core.Shape")


class Shape(object):

    """
    The Shape Class includes all plotting, GUI functionality and export functions
    related to the Shapes.
    """
    # only need default arguments here because of the change of usage with
    # super in QGraphicsItem

    def __init__(self, nr=-1, closed=True, parentEntity=None, geos=[]):
        if nr == -1:
            return

        self.type = "Shape"
        self.nr = nr
        self.closed = closed
        self.cut_cor = 40
        self.parentEntity = parentEntity
        self.parentLayer = None
        self.geos = Geos(geos)

        self.cw = True

        self.stmove = None

        self.send_to_TSP = g.config.vars.Route_Optimisation['default_TSP']

        self.selected = False
        self.disabled = False
        self.allowedToChange = True

        # preset defaults
        self.axis3_start_mill_depth = g.config.vars.Depth_Coordinates[
            'axis3_start_mill_depth']
        self.axis3_slice_depth = g.config.vars.Depth_Coordinates[
            'axis3_slice_depth']
        self.axis3_mill_depth = g.config.vars.Depth_Coordinates[
            'axis3_mill_depth']
        self.f_g1_plane = g.config.vars.Feed_Rates['f_g1_plane']
        self.f_g1_depth = g.config.vars.Feed_Rates['f_g1_depth']
        # Parameters for drag knife
        self.drag_angle = radians(
            g.config.vars.Drag_Knife_Options['drag_angle'])

    def __str__(self):
        """
        Standard method to print the object
        @return: A string
        """

        logger.debug(self.closed)
        return "\nnr:          %i" % self.nr +\
               "\nclosed:      %s" % self.closed +\
               "\ngeos:        %s" % self.geos

    def tr(self, string_to_translate):
        """
        Translate a string using the QCoreApplication translation framework
        @param: string_to_translate: a unicode string
        @return: the translated unicode string if it was possible to translate
        """
        return str(QtCore.QCoreApplication.translate("Shape",
                                                           string_to_translate))

    def setSelected(self, flag=False):
        self.selected = flag

    def isSelected(self):
        return self.selected

    def setDisable(self, flag=False):
        self.disabled = flag

    def isDisabled(self):
        return self.disabled

    def setToolPathOptimized(self, flag=False):
        self.send_to_TSP = flag

    def isToolPathOptimized(self):
        return self.send_to_TSP

    def isDirectionOfGeosCCW(self, geos):
        # By calculating the area of the shape
        start = geos.abs_el(0).get_start_end_points(True)
        summe = 0.0
        for geo in geos.abs_iter():
            if isinstance(geo, LineGeo):
                end = geo.get_start_end_points(False)
                summe += (start.x + end.x) * (end.y - start.y)
                start = end
            elif isinstance(geo, ArcGeo):
                segments = 10
                for i in range(1, segments + 1):
                    end = geo.get_point_from_start(i, segments)
                    summe += (end.x + start.x) * (end.y - start.y)
                    start = end
        if not self.closed:
            # if shape is not closed... simply treat it as closed
            end = geos.abs_el(0).get_start_end_points(True)
            summe += (end.x + start.x) * (end.y - start.y)

        if summe == 0:  # inconclusive
            logger.debug(
                self.tr("Shoelace method cannot (directly) be applied to this shape"))
            # lets take it clock wise with relation to the workpiece zero

            start = geos.abs_el(0).get_start_end_points(True)
            # get the farthest end point with relation to the start
            end = start
            distance2 = 0
            for geo in geos.abs_iter():
                pos_end = geo.get_start_end_points(False)
                pos_distance2 = (start - pos_end).length_squared()
                if pos_distance2 > distance2:
                    end = pos_end
                    distance2 = pos_distance2
            direction = start.to3D().cross_product(end.to3D()).z
            # start and end are aligned wrt to wp zero
            if -1e-5 < direction < 1e-5:
                direction = start.length_squared() - end.length_squared()
            summe = direction
        return summe > 0.0

    def AnalyseAndOptimize(self):
        self.setNearestStPoint(Point())
        logger.debug(
            self.tr("Analysing the shape for CW direction Nr: %s" % self.nr))

        if self.isDirectionOfGeosCCW(self.geos):
            self.reverse()
            logger.debug(self.tr("Had to reverse the shape to be CW"))
        self.cw = True

    def setNearestStPoint(self, stPoint):
        if self.closed:
            logger.debug(self.tr("Clicked Point: %s" % stPoint))
            start = self.get_start_end_points(True)
            logger.debug(self.tr("Old Start Point: %s" % start))

            min_geo_nr, _ = min(enumerate(self.geos.abs_iter()),
                                key=lambda geo:
                                geo[1].get_start_end_points(True).distance(stPoint))

            # Overwrite the geometries in changed order.
            self.geos = Geos(self.geos[min_geo_nr:] + self.geos[:min_geo_nr])

            start = self.get_start_end_points(True)
            logger.debug(self.tr("New Start Point: %s" % start))
        elif len (self.geos) > 0:
            # if shape is not closed, we can just select between start and end points
            dist1 = self.geos.abs_el(0).get_start_end_points(True).distance(stPoint)
            dist2 = self.geos.abs_el(-1).get_start_end_points(False).distance(stPoint)
            if dist2 < dist1:
                self.reverse()

    def reverse(self, geos=None):
        if not geos:
            geos = self.geos
        geos.reverse()
        for geo in geos:
            geo.reverse()
        self.cw = not self.cw

    def switch_cut_cor(self):
        """
        Switches the cutter direction between 41 and 42.

        G41 = Tool radius compensation left.
        G42 = Tool radius compensation right
        """
        if self.cut_cor == 41:
            self.cut_cor = 42
        elif self.cut_cor == 42:
            self.cut_cor = 41

    def append(self, geo):
        geo.make_abs_geo(self.parentEntity)
        self.geos.append(geo)

    def get_start_end_points_physical(self, start_point=None, angles=None):
        """
        With multiple slices end point could be start point.
        e.g. useful for the optimal rout etc
        """
        if start_point or self.closed:
            return self.get_start_end_points(start_point, angles)
        else:
            max_slice = max(self.axis3_slice_depth,
                            self.axis3_mill_depth -
                            self.axis3_start_mill_depth)
            if max_slice == 0:
                end_should_be_start = True
            else:
                end_should_be_start = (
                    self.axis3_start_mill_depth - self.axis3_mill_depth) // max_slice % 2 == 0
            if not end_should_be_start:
                return self.get_start_end_points(start_point, angles)
            else:
                start_stuff = self.get_start_end_points(True, angles)
                if angles is False:
                    end_stuff = start_stuff[0], -start_stuff[1]
                else:
                    end_stuff = start_stuff
                if start_point is None:
                    return start_stuff, end_stuff
                else:
                    return end_stuff

    def get_start_end_points(self, start_point=None, angles=None):
        if start_point is None:
            return (self.geos.abs_el(0).get_start_end_points(True, angles),
                    self.geos.abs_el(-1).get_start_end_points(False, angles))
        elif start_point:
            return self.geos.abs_el(0).get_start_end_points(True, angles)
        else:
            return self.geos.abs_el(-1).get_start_end_points(False, angles)

    def make_path(self, drawHorLine, drawVerLine):

        for geo in self.geos.abs_iter():
            drawVerLine(self, geo.get_start_end_points(True))

            geo.make_path(self, drawHorLine)

        if not self.closed:
            drawVerLine(self, geo.get_start_end_points(False))

        self.calc_bounding_box()

    def calc_bounding_box(self):
        """
        Calculated the BoundingBox of the geometry and saves it into self.BB
        """
        self.BB = self.geos.abs_el(0).BB
        for geo in self.geos.abs_iter():
            self.BB = self.BB.joinBB(geo.BB)

    def make_shape_ccw(self):
        """
        This method is called after the shape has been generated before it gets
        plotted to change all shape direction to a CW shape.
        """

        if not(self.closed):
            return

        # Optimization for closed shapes
        # Start value for the first sum

        if self.isDirectionOfGeosCCW(self.geos):
            self.reverse()
            logger.debug(self.tr("Had to reverse the shape to be CW"))
        self.cw = True

    def isHit(self, xy, tol):
        if self.BB.Ps.x - tol <= xy.x <= self.BB.Pe.x + tol\
                and self.BB.Ps.y - tol <= xy.y <= self.BB.Pe.y + tol:
            for geo in self.geos.abs_iter():
                if geo.isHit(self, xy, tol):
                    return True
        return False

    def Write_GCode_for_geo(self, geo, PostPro):
        # Used to remove zero length geos. If not, arcs can become a full
        # circle
        post_dec = PostPro.vars.Number_Format["post_decimals"]
        if isinstance(geo, HoleGeo) or\
           round(geo.Ps.x, post_dec) != round(geo.Pe.x, post_dec) or\
           round(geo.Ps.y, post_dec) != round(geo.Pe.y, post_dec) or\
           isinstance(geo, ArcGeo) and geo.length > 0.5 * 0.1 ** post_dec * pi:
            return geo.Write_GCode(PostPro)
        else:
            return ""

    def Write_GCode(self, PostPro):
        """
        This method returns the string to be exported for this shape, including
        the defined start and end move of the shape.
        @param PostPro: this is the Postprocessor class including the methods
        to export
        """
        if g.config.machine_type == 'drag_knife':
            return self.Write_GCode_Drag_Knife(PostPro)

        prv_cut_cor = self.cut_cor
        if (self.cut_cor != 40 and
                not g.config.vars.Cutter_Compensation["done_by_machine"]):
            self.cut_cor = 40
            new_geos = Geos(self.stmove.geos[1:])
        else:
            new_geos = self.geos

        # If there is nothing to export
        if len(new_geos) == 0:
            return ""

        new_geos = PostPro.breaks.getNewGeos(new_geos)
        # initialisation of the string
        exstr = ""

        # Get the mill settings defined in the GUI
        safe_retract_depth = self.parentLayer.axis3_retract
        safe_margin = self.parentLayer.axis3_safe_margin

        max_slice = self.axis3_slice_depth
        workpiece_top_Z = self.axis3_start_mill_depth
        # We want to mill the piece, even for the first pass, so remove one
        # "slice"
        initial_mill_depth = workpiece_top_Z - abs(max_slice)
        depth = self.axis3_mill_depth
        f_g1_plane = self.f_g1_plane
        f_g1_depth = self.f_g1_depth

        # Save the initial Cutter correction in a variable
        has_reversed = False

        # If the Output Format is DXF do not perform more then one cut.
        if PostPro.vars.General["output_type"] == 'dxf':
            depth = max_slice

        if max_slice == 0:
            logger.error(self.tr("ERROR: Z infeed depth is null!"))

        if initial_mill_depth < depth:
            logger.warning(self.tr(
                "WARNING: initial mill depth (%i) is lower than end mill depth (%i). Using end mill depth as final depth.") % (
                initial_mill_depth, depth))

            # Do not cut below the depth.
            initial_mill_depth = depth

        mom_depth = initial_mill_depth

        # Move the tool to the start.
        exstr += self.stmove.geos.abs_el(0).Write_GCode(PostPro)

        # Add string to be added before the shape will be cut.
        exstr += PostPro.write_pre_shape_cut()

        # Cutter radius compensation when G41 or G42 is on, AND cutter
        # compensation option is set to be done outside the piece
        if self.cut_cor != 40 and PostPro.vars.General["cc_outside_the_piece"]:
            exstr += PostPro.set_cut_cor(self.cut_cor)

            exstr += PostPro.chg_feed_rate(f_g1_plane)
            exstr += self.stmove.geos.abs_el(1).Write_GCode(PostPro)
            exstr += self.stmove.geos.abs_el(2).Write_GCode(PostPro)

        exstr += PostPro.rap_pos_z(
            workpiece_top_Z + abs(safe_margin))  # Compute the safe margin from the initial mill depth
        exstr += PostPro.chg_feed_rate(f_g1_depth)
        exstr += PostPro.lin_pol_z(mom_depth)
        exstr += PostPro.chg_feed_rate(f_g1_plane)

        # Cutter radius compensation when G41 or G42 is on, AND cutter
        # compensation option is set to be done inside the piece
        if self.cut_cor != 40 and not PostPro.vars.General["cc_outside_the_piece"]:
            exstr += PostPro.set_cut_cor(self.cut_cor)

            exstr += self.stmove.geos.abs_el(1).Write_GCode(PostPro)
            exstr += self.stmove.geos.abs_el(2).Write_GCode(PostPro)

        # Write the geometries for the first cut
        for geo in new_geos.abs_iter():
            exstr += self.Write_GCode_for_geo(geo, PostPro)

        # Turning the cutter radius compensation
        if self.cut_cor != 40 and PostPro.vars.General["cancel_cc_for_depth"]:
            exstr += PostPro.deactivate_cut_cor()

        # Numbers of loops
        snr = 0
        # Loops for the number of cuts
        while mom_depth > depth and max_slice != 0.0:
            snr += 1
            mom_depth = mom_depth - abs(max_slice)
            if mom_depth < depth:
                mom_depth = depth

            # Erneutes Eintauchen
            exstr += PostPro.chg_feed_rate(f_g1_depth)
            exstr += PostPro.lin_pol_z(mom_depth)
            exstr += PostPro.chg_feed_rate(f_g1_plane)

            # If it is not a closed contour
            if not self.closed:
                self.reverse(new_geos)
                self.switch_cut_cor()
                # switch the "reversed" state (in order to restore it at the
                # end)
                has_reversed = not has_reversed

                # If cutter radius compensation is turned on. Turn it off - because some interpreters cannot handle
                # a switch
                if self.cut_cor != 40 and not PostPro.vars.General["cancel_cc_for_depth"]:
                    exstr += PostPro.deactivate_cut_cor()
                    exstr += PostPro.set_cut_cor(self.cut_cor)


            # If cutter correction is enabled
            if self.cut_cor != 40 and PostPro.vars.General["cancel_cc_for_depth"]:
                exstr += PostPro.set_cut_cor(self.cut_cor)

            for geo in new_geos.abs_iter():
                exstr += self.Write_GCode_for_geo(geo, PostPro)

            # Turning off the cutter radius compensation if needed
            if self.cut_cor != 40 and PostPro.vars.General["cancel_cc_for_depth"]:
                exstr += PostPro.deactivate_cut_cor()

        # Do the tool retraction
        exstr += PostPro.chg_feed_rate(f_g1_depth)
        exstr += PostPro.lin_pol_z(workpiece_top_Z + abs(safe_margin))
        exstr += PostPro.rap_pos_z(safe_retract_depth)

        # If cutter radius compensation is turned on.
        if self.cut_cor != 40 and not PostPro.vars.General["cancel_cc_for_depth"]:
            exstr += PostPro.deactivate_cut_cor()

        # Initial value of direction restored if necessary
        if has_reversed:
            self.reverse(new_geos)
            self.switch_cut_cor()

        self.cut_cor = prv_cut_cor

        # Add string to be added before the shape will be cut.
        exstr += PostPro.write_post_shape_cut()

        return exstr

    def Write_GCode_Drag_Knife(self, PostPro):
        """
        This method returns the string to be exported for this shape, including
        the defined start and end move of the shape. This function is used for
        Drag Knife cutting machine only.
        @param PostPro: this is the Postprocessor class including the methods
        to export
        """

        # initialisation of the string
        exstr = ""

        # Get the mill settings defined in the GUI
        safe_retract_depth = self.parentLayer.axis3_retract
        safe_margin = self.parentLayer.axis3_safe_margin

        workpiece_top_Z = self.axis3_start_mill_depth
        f_g1_plane = self.f_g1_plane
        f_g1_depth = self.f_g1_depth

        """
        Cutting in slices is not supported for Swivel Knife tool. All is cut at once.
        """
        mom_depth = self.axis3_mill_depth
        drag_depth = self.axis3_slice_depth

        # Move the tool to the start.
        exstr += self.stmove.geos.abs_el(0).Write_GCode(PostPro)

        # Add string to be added before the shape will be cut.
        exstr += PostPro.write_pre_shape_cut()

        # Move into workpiece and start cutting into Z
        exstr += PostPro.rap_pos_z(
            workpiece_top_Z + abs(safe_margin))  # Compute the safe margin from the initial mill depth
        exstr += PostPro.chg_feed_rate(f_g1_depth)

        # Write the geometries for the first cut
        if isinstance(self.stmove.geos.abs_el(1), ArcGeo):
            if self.stmove.geos.abs_el(1).drag:
                exstr += PostPro.lin_pol_z(drag_depth)
                drag = True
            else:
                exstr += PostPro.lin_pol_z(mom_depth)
                drag = False
        else:
            exstr += PostPro.lin_pol_z(mom_depth)
            drag = False
        exstr += PostPro.chg_feed_rate(f_g1_plane)

        exstr += self.stmove.geos.abs_el(1).Write_GCode(PostPro)

        for geo in Geos(self.stmove.geos[2:]).abs_iter():
            if isinstance(geo, ArcGeo):
                if geo.drag:
                    exstr += PostPro.chg_feed_rate(f_g1_depth)
                    exstr += PostPro.lin_pol_z(drag_depth)
                    exstr += PostPro.chg_feed_rate(f_g1_plane)
                    drag = True
                elif drag:
                    exstr += PostPro.chg_feed_rate(f_g1_depth)
                    exstr += PostPro.lin_pol_z(mom_depth)
                    exstr += PostPro.chg_feed_rate(f_g1_plane)
                    drag = False
            elif drag:
                exstr += PostPro.chg_feed_rate(f_g1_depth)
                exstr += PostPro.lin_pol_z(mom_depth)
                exstr += PostPro.chg_feed_rate(f_g1_plane)
                drag = False

            exstr += self.Write_GCode_for_geo(geo, PostPro)

        # Do the tool retraction
        exstr += PostPro.chg_feed_rate(f_g1_depth)
        exstr += PostPro.lin_pol_z(workpiece_top_Z + abs(safe_margin))
        exstr += PostPro.rap_pos_z(safe_retract_depth)

        # Add string to be added before the shape will be cut.
        exstr += PostPro.write_post_shape_cut()

        return exstr

    def join_colinear_lines(self):
        """
        This function is called to search for colinear connected lines an joins 
        them if there are any
        """
        # Do only if more then 2 geometies
        if len(self.geos) < 2:
            return

        new_geos = []
        for i in range(len(self.geos)):
            if len(new_geos) == 0:
                new_geos.append(self.geos[i])
                continue

            geo1 = new_geos[-1]
            geo2 = self.geos[i]

            if isinstance(geo1, LineGeo) and isinstance(geo2, LineGeo):
                # Remove first geometry and add result of joined geometries. Required
                # Cause the join will give back the last 2 geometries.
                new_geos.pop()
                new_geos += geo1.join_colinear_line(geo2)
            elif isinstance(geo1, ArcGeo) or isinstance(geo2, ArcGeo):
                new_geos.append(geo2)

            # If start end End Point are the same remove geometry
            if new_geos[-1].Ps == new_geos[-1].Pe:
                # Note that this can make new_geos absolutely empty!
                new_geos.pop()

        # For closed polylines check if the first and last items are colinear
        if self.closed and (len (new_geos) > 1):
            geo1 = new_geos[-1]
            geo2 = new_geos[0]
            if isinstance(geo1, LineGeo) and isinstance(geo2, LineGeo):
                joined_geos = geo1.join_colinear_line(geo2)

                # If they are joined replace first item by joined and remove
                # last one
                if len(joined_geos) == 1:
                    new_geos[0] = joined_geos[0]
                    new_geos.pop()

        self.geos = new_geos


class Geos(list):

    def __init__(self, *args):
        list.__init__(self, *args)

    def abs_iter(self):
        for geo in list.__iter__(self):
            yield geo.abs_geo if geo.abs_geo else geo


    def abs_el(self, element):
        return self[element].abs_geo if self[element].abs_geo else self[element]



# -*- coding: utf-8 -*-

############################################################################
#
#   Copyright (C) 2010-2015
#    Christian Kohlöffel
#    Jean-Paul Schouwstra
#
#   This file is part of DXF2GCODE.
#
#   DXF2GCODE is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   DXF2GCODE is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with DXF2GCODE.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################

"""
All global constants are initialized in this module.
They are used in the other modules.

see http://code.activestate.com/recipes/65207/ for module const

@purpose:  initialization of the global constants used within the other modules.
"""

import logging

import platform

from PyQt5 import QtCore

# Global Variables
APPNAME = "DXF2GCODE"
VERSION = "Py%s PyQt%s" % (platform.python_version(), QtCore.PYQT_VERSION_STR)

DATE     =  "$Date: Fri Oct 25 20:45:56 2019 +0200 $"
REVISION =  "$Revision: cd19f2420da8d0845363d0fb120edc130ddf50a5 $"
AUTHOR   = u"$Author: Damian Wrobel <dwrobel@ertelnet.rybnik.pl> $"

CONFIG_EXTENSION = '.cfg'
PY_EXTENSION = '.py'
PROJECT_EXTENSION = '.d2g'

# Rename unreadable config/varspace files to .bad
BAD_CONFIG_EXTENSION = '.bad'
DEFAULT_CONFIG_DIR = 'config'
DEFAULT_POSTPRO_DIR = 'postpro_config'

# log related
DEFAULT_LOGFILE = 'dxf2gcode.log'
STARTUP_LOGLEVEL = logging.DEBUG
# PRT = logging.INFO
